@@ -4,13 +4,73 @@ extern crate glob;
 use std::env;
 use std::path::{Path, PathBuf};
 
-fn main() {
-    let internals_include_path = &std::env::var("DEP_PQCRYPTO_INTERNALS_INCLUDEPATH").unwrap();
-    let common_dir = Path::new("pqclean/common");
+macro_rules! build_clean {
+    ($variant:expr) => {
+        let internals_include_path = &std::env::var("DEP_PQCRYPTO_INTERNALS_INCLUDEPATH").unwrap();
+        let common_dir = Path::new("pqclean/common");
 
+        let mut builder = cc::Build::new();
+        let target_dir: PathBuf = ["pqclean", "crypto_sign", $variant, "clean"]
+            .iter()
+            .collect();
+
+        let scheme_files = glob::glob(target_dir.join("*.c").to_str().unwrap()).unwrap();
+
+        builder
+            .include(internals_include_path)
+            .include(&common_dir)
+            .include(target_dir)
+            .files(
+                scheme_files
+                    .into_iter()
+                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
+            );
+        builder.compile(format!("{}_clean", $variant).as_str());
+    };
+}
+
+macro_rules! build_avx2 {
+    ($variant:expr) => {
+        let internals_include_path = &std::env::var("DEP_PQCRYPTO_INTERNALS_INCLUDEPATH").unwrap();
+        let common_dir = Path::new("pqclean/common");
+
+        let mut builder = cc::Build::new();
+        let target_dir: PathBuf = ["pqclean", "crypto_sign", $variant, "avx2"]
+            .iter()
+            .collect();
+
+        let scheme_files = glob::glob(target_dir.join("*.[csS]").to_str().unwrap()).unwrap();
+        if cfg!(target_env = "msvc") {
+            builder.flag("/arch:AVX2");
+        } else {
+            builder
+                .flag("-mavx2")
+                .flag("-mbmi2")
+                .flag("-mbmi")
+                .flag("-maes")
+                .flag("-mpopcnt")
+                .flag("-mpclmul");
+        }
+
+        builder
+            .include(internals_include_path)
+            .include(&common_dir)
+            .include(target_dir)
+            .files(
+                scheme_files
+                    .into_iter()
+                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
+            );
+        builder.compile(format!("{}_avx2", $variant).as_str());
+    };
+}
+
+fn main() {
     #[allow(unused_variables)]
     let avx2_enabled = env::var("CARGO_FEATURE_AVX2").is_ok();
     #[allow(unused_variables)]
+    let aes_enabled = env::var("CARGO_FEATURE_AES").is_ok();
+    #[allow(unused_variables)]
     let target_arch = env::var("CARGO_CFG_TARGET_ARCH").unwrap();
     #[allow(unused_variables)]
     let target_os = env::var("CARGO_CFG_TARGET_OS").unwrap();
@@ -19,103 +79,17 @@ fn main() {
     #[allow(unused_variables)]
     let is_macos = target_os == "macos";
 
-    {
-        let mut builder = cc::Build::new();
-        let target_dir: PathBuf = ["pqclean", "crypto_sign", "falcon-512", "clean"]
-            .iter()
-            .collect();
-        let scheme_files = glob::glob(target_dir.join("*.c").to_str().unwrap()).unwrap();
-        builder
-            .include(internals_include_path)
-            .include(&common_dir)
-            .include(target_dir)
-            .files(
-                scheme_files
-                    .into_iter()
-                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
-            );
-        builder.compile("falcon-512_clean");
+    build_clean!("falcon-512");
+    if avx2_enabled && target_arch == "x86_64" {
+        build_avx2!("falcon-512");
+    }
+    build_clean!("falcon-1024");
+    if avx2_enabled && target_arch == "x86_64" {
+        build_avx2!("falcon-1024");
     }
 
     if avx2_enabled && target_arch == "x86_64" {
-        let target_dir: PathBuf = ["pqclean", "crypto_sign", "falcon-512", "avx2"]
-            .iter()
-            .collect();
-        let scheme_files = glob::glob(target_dir.join("*.[csS]").to_str().unwrap()).unwrap();
-        let mut builder = cc::Build::new();
-
-        if cfg!(target_env = "msvc") {
-            builder.flag("/arch:AVX2");
-        } else {
-            builder
-                .flag("-mavx2")
-                .flag("-mbmi2")
-                .flag("-mbmi")
-                .flag("-maes")
-                .flag("-mpopcnt")
-                .flag("-mpclmul");
-        }
-        builder
-            .include(internals_include_path)
-            .include(&common_dir)
-            .include(target_dir)
-            .files(
-                scheme_files
-                    .into_iter()
-                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
-            )
-            .compile("falcon-512_avx2");
-    }
-    {
-        let mut builder = cc::Build::new();
-        let target_dir: PathBuf = ["pqclean", "crypto_sign", "falcon-1024", "clean"]
-            .iter()
-            .collect();
-        let scheme_files = glob::glob(target_dir.join("*.c").to_str().unwrap()).unwrap();
-        builder
-            .include(internals_include_path)
-            .include(&common_dir)
-            .include(target_dir)
-            .files(
-                scheme_files
-                    .into_iter()
-                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
-            );
-        builder.compile("falcon-1024_clean");
-    }
-
-    if avx2_enabled && target_arch == "x86_64" {
-        let target_dir: PathBuf = ["pqclean", "crypto_sign", "falcon-1024", "avx2"]
-            .iter()
-            .collect();
-        let scheme_files = glob::glob(target_dir.join("*.[csS]").to_str().unwrap()).unwrap();
-        let mut builder = cc::Build::new();
-
-        if cfg!(target_env = "msvc") {
-            builder.flag("/arch:AVX2");
-        } else {
-            builder
-                .flag("-mavx2")
-                .flag("-mbmi2")
-                .flag("-mbmi")
-                .flag("-maes")
-                .flag("-mpopcnt")
-                .flag("-mpclmul");
-        }
-        builder
-            .include(internals_include_path)
-            .include(&common_dir)
-            .include(target_dir)
-            .files(
-                scheme_files
-                    .into_iter()
-                    .map(|p| p.unwrap().to_string_lossy().into_owned()),
-            )
-            .compile("falcon-1024_avx2");
-    }
-
-    // Print enableing flag for AVX2 implementation
-    if avx2_enabled && target_arch == "x86_64" {
+        // Print enableing flag for AVX2 implementation
         println!("cargo:rustc-cfg=enable_avx2");
     }
 }
